﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Interactivity;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Extensibility;

namespace NavAddin
{

	public interface INavTool
	{
		void Check();
		void Uncheck();
	}

	[Export(typeof(Behavior<Map>))]
	[DisplayName("Tool Manager Behavior")]
	public class ToolBehavior : Behavior<Map>
	{

		public static ToolBehavior Current { get; private set; }

		private Dictionary<string, INavTool> _Tools = new Dictionary<string, INavTool>();
		private string _Tool = "";
		private ucMouseCursor _Cursor = new ucMouseCursor();

		#region Behavior Overrides
		protected override void OnAttached()
		{
			base.OnAttached();
			Current = this;
			_Cursor.Visibility = Visibility.Collapsed;
			Map MyMap = MapApplication.Current.Map;
			FrameworkElement parent = VisualTreeHelper.GetParent(MyMap) as FrameworkElement;
			if (!(parent is Grid))
			{
				MessageBox.Show("ToolBehavior requires that the Map is a child of a Grid.", "ToolBehavior", MessageBoxButton.OK);
				return;
			}
			((Grid)parent).Children.Add(_Cursor);
			// This is probably not necessary
			//_Cursor.Width = MyMap.Width; 
			//_Cursor.Height = MyMap.Height;
			//_Cursor.HorizontalAlignment = MyMap.HorizontalAlignment;
			//_Cursor.VerticalAlignment = MyMap.VerticalAlignment;
			Grid.SetColumn(_Cursor, Grid.GetColumn(MyMap));
			Grid.SetColumnSpan(_Cursor, Grid.GetColumnSpan(MyMap));
			Grid.SetRow(_Cursor, Grid.GetRow(MyMap));
			Grid.SetRowSpan(_Cursor, Grid.GetRowSpan(MyMap));
			MyMap.MouseMove += new System.Windows.Input.MouseEventHandler(MyMap_MouseMove);
			MyMap.MouseEnter += new System.Windows.Input.MouseEventHandler(MyMap_MouseEnter);
			MyMap.MouseLeave += new System.Windows.Input.MouseEventHandler(MyMap_MouseLeave);
		}
		#endregion

		#region Public Methods
		public void RegisterTool(string MyKey, INavTool MyTool, MouseCursorDef MyCursor = null)
		{

			// MyKey must be a unique string value <> ""
			// Use the same key to change the cursor

			if ((MyKey == null) || (MyKey == ""))
				return;
			_Tools[MyKey] = MyTool;
			_Cursor.RegisterCursor(MyKey, MyCursor);

		}

		public void SetTool(string MyKey)
		{

			// Use "" to deactivate all tools and cursor

			_Tool = "";
			foreach (string sKey in _Tools.Keys)
			{
				INavTool pTool = _Tools[sKey];
				if (sKey != MyKey)
					pTool.Uncheck();
				else
					_Tool = MyKey;
			}
			if (_Tool != "")
			{
				INavTool pTool = _Tools[_Tool];
				pTool.Check();
			}
			_Cursor.SetCursor(_Tool);
			Map MyMap = MapApplication.Current.Map;
			if (_Tool == "")
				MyMap.Cursor = Cursors.Arrow;
			else
				MyMap.Cursor = Cursors.None;
				
		}
		#endregion

		#region Mouse Event Handlers
		private void MyMap_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if (_Tool == "")
				return;
			Map MyMap = MapApplication.Current.Map;
			_Cursor.MoveTo(e.GetPosition(MyMap));
		}

		private void MyMap_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if (_Tool != "")
				_Cursor.SetCursor(_Tool);
		}

		private void MyMap_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
		{
			if (_Tool != "")
				_Cursor.SetCursor("");
		}
		#endregion

	}
}
